// *****************************************************************
// Filename:    MathTools.h
// Copyright:   Kai Welke, Chair Prof. Dillmann (IAIM),
//              Institute for Computer Science and Engineering (CSE),
//              University of Karlsruhe. All rights reserved.
// Author:      Kai Welke
// Date:        12.06.2007
// *****************************************************************

#ifndef _MATH_TOOLS_H_
#define _MATH_TOOLS_H_

// *****************************************************************
// defines
// *****************************************************************
#include <math.h>
#ifndef M_PI
#define M_PI 3.141592653589793
#endif 

// *****************************************************************
// includes
// *****************************************************************
#include "Structs.h"
#include <Math/Math3d.h>
#include <Math/Math2d.h>

#include <Eigen/Eigen>

// *****************************************************************
// class MathTools
// *****************************************************************
class MathTools
{
public:
	// transformation of coordinates and paths
	static TTransform determinePathTransformation(TPath* p1, TPath* p2);
	static TPath* transformPath(TTransform transform, TPath* pPath);
	static TPath* inverseTransformPath(TTransform transform, TPath* pPath);

	static TTransform determineTransformation(TSphereCoord start1, TSphereCoord start2, TSphereCoord end1, TSphereCoord end2);
	static TSphereCoord transform(TSphereCoord sc, TTransform transform);
	static TSphereCoord inverseTransform(TSphereCoord sc, TTransform transform);

	static void determineTransformationAlphaBeta(TSphereCoord sc1, TSphereCoord sc2, float& fAlpha, float& fBeta, Vec3d& betaAxis);
	static TSphereCoord transformAlphaBeta(TSphereCoord sc, float fAlpha, float fBeta, Vec3d betaAxis);
	static TSphereCoord inverseTransformAlphaBeta(TSphereCoord sc, float fAlpha, float fBeta, Vec3d betaAxis);

	// calculations on sphere coordimates
	static float CalculateCrossProductFromDifference(TSphereCoord p1, TSphereCoord p2, TSphereCoord p3);
	static float getDistanceOnArc(TSphereCoord p1, TSphereCoord p2);
	static float getDistance(TSphereCoord p1, TSphereCoord p2, float fRadius);

	// extended 2d math
	static bool IntersectLines(Vec2d p1, Vec2d m1, Vec2d p2, Vec2d m2, Vec2d& res);

	// extended 3d math
	static Vec3d dropAPerpendicular(Vec3d point, Vec3d linepoint, Vec3d linevector);
	static float AngleAndDirection(Vec3d vector1, Vec3d vector2, Vec3d normal);

	// coorrdinate conversion 
	static void convert(TSphereCoord in, Vec3d& out);
    static void convert(TSphereCoord in, Eigen::Vector3f& out);
	static void convert(Vec3d in, TSphereCoord& out);
    static void convert(Eigen::Vector3f in, TSphereCoord& out);
	static void convertWithDistance(TSphereCoord in, Vec3d& out);


};

#endif /* _MATH_TOOLS_H_ */
