/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarX::Component::ObjectExaminerGuiPlugin
* @author     Nikolaus Vahrenkamp ( vahrenkamp at kit dot edu)
* @copyright  2012
* @license    http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License

*/

#ifndef _ARMARXGUI_PLUGINS_MMMPlayerWIDGET_H
#define _ARMARXGUI_PLUGINS_MMMPlayerWIDGET_H

/* ArmarX headers */
#include "ui_MMMPlayerGuiPlugin.h"
#include <ArmarXCore/core/Component.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXCore/observers/Observer.h>

/* Qt headers */
#include <QtGui/QMainWindow>
#include <QtCore/QTimer>
#include <RobotComponents/interface/components/MMMPlayerInterface.h>

#include <string>
#include <QLayout>
#include <QFileDialog>
#include <QListWidget>

namespace armarx
{
    class MMMPlayerConfigDialog;

    /*!
      \class MMMPlayerGuiPlugin
      \brief This plugin provides a widget with which the MMMPlayer can be controlled.

      \see MMMPlayerWidget
      */
    class MMMPlayerGuiPlugin :
            public ArmarXGuiPlugin
    {
    public:
        MMMPlayerGuiPlugin();
        QString getPluginName()
        {
            return "MMMPlayerGuiPlugin";
        }
    };

    /*!
      \class MMMPlayerWidget
      \brief With this widget the MMMPlayer can be controlled.

      \ingroup RobotAPI-ArmarXGuiPlugins ArmarXGuiPlugins
      \see MMMPlayerGuiPlugin
      */
    class MMMPlayerWidget :
            public ArmarXComponentWidgetController
    {
        Q_OBJECT
    public:
        MMMPlayerWidget();
        ~MMMPlayerWidget()
        {}

        // inherited from Component
        virtual void onInitComponent();
        virtual void onConnectComponent();
        virtual void onExitComponent();
        virtual void onDisconnectComponent();

        // inherited of ArmarXWidget
        virtual QString getWidgetName() const { return "RobotControl.MMMPlayerGUI"; }
        virtual QIcon getWidgetIcon() const { return QIcon("://icons/haptic-hand.png"); }

//        QPointer<QDialog> getConfigDialog(QWidget* parent = 0);
        virtual void loadSettings(QSettings * settings);
        virtual void saveSettings(QSettings * settings);
//        void configured();

    signals:
 //       void doUpdateDisplayWidgets();

    private slots:
//        void updateDisplayWidgets();

        void on_startButton_clicked();

        void on_pauseButton_clicked();

        void on_stopButton_clicked();

        void setFrameNumber(int);

        void configFileSelected();

        void updateSlider();

        void on_loopPlayback_toggled(bool state);

        void jointListChanged(QListWidgetItem joint);
    protected:
        void connectSlots();

        std::string configFile;
        Ui::MMMPlayerGuiPlugin ui;
        MMMPlayerInterfacePrx MMMPlayer;

    private:

//        void createMatrixWidgets();

    private:

        QPointer<QWidget> __widget;
        QPointer<MMMPlayerConfigDialog> dialog;
        QFileDialog* fileDialog2;
        QListWidget* jointList;

        QTimer* updateTimer;
    };
    //typedef boost::shared_ptr<MMMPlayerWidget> MMMPlayerGuiPluginPtr;
}

#endif
